#!/bin/bash
# This is the up.time linux agent
#
# localization issue
export LANG=en_US
export LC_COLLATE=en_US

TMPDIR="/opt/uptime-agent/tmp"
UPTIME_DIR="/opt/uptime-agent/bin"
SARBIN=$UPTIME_DIR/sar
#SARBIN=/usr/bin/sar
IOSTATBIN=$UPTIME_DIR/iostat
#IOSTATBIN=/usr/bin/iostat
if [ -x /bin/gawk ]; then
AWKBIN=/bin/gawk
else
  if [ -x /bin/awk ]; then
    AWKBIN=/bin/awk
  else
    AWKBIN=/usr/bin/awk
  fi
fi
VMSTATBIN=/usr/bin/vmstat
NETSTATBIN=/bin/netstat
JOINBIN=/usr/bin/join
DATESTR=`date '+%Y%m%d%H%M'`
HOST=`uname -n`
DFBIN="df -lk"
OSRELEASE=`cat /proc/sys/kernel/osrelease | awk -F. '{print $1"."$2}'`
LOGFILE=$UPTIME_DIR/uptmagnt.log
TCPINFO_TMP=$TMPDIR/.tcp.uptm
TLIST_TMP=$TMPDIR/.tlist.uptm
IOSTATD_TMP=$TMPDIR/.iostatd.uptm
IOSTATDX_TMP=$TMPDIR/.iostatdx.uptm
SADC_DISKD_TMP=$TMPDIR/.sadc_diskd.uptm
SADC_DISKDX_TMP=$TMPDIR/.sadc_diskdx.uptm

log() {
	echo "$DATESTR $1" >> $LOGFILE
}

netstat_disp() {
# We have to answer with the following fields:
# A timestamp, interface name, bytes/s in, bytes/s out, collisions, receive errors per second, transmit errors per second
# $SARBIN -n DEV -n EDEV 1 | grep Average | egrep -v 'lo|IFACE' | $AWKBIN '{print $2" "$3" "$4" "$5" "$6}' | while read IF_NAME DUMMY DUMMY2 RXBYTES TXBYTES

$SARBIN -n DEV -n EDEV 1 | grep Average | egrep -v 'lo|IFACE' | sort -s -k 2b,2 -k 2,2n | $AWKBIN 'BEGIN { i=0 } {print $2" "i++" "$3" "$4" "$5" "$6}' | while read IF_NAME DUMMY DUMMY1 DUMMY2 RXBYTES TXBYTES
do
        read DUMMY DUMMY1 RXERRORS TXERRORS COLLS DUMMY2
 	/sbin/ifconfig | awk -v DATESTR=$DATESTR -v IF_NAME=$IF_NAME -v RXBYTES=$RXBYTES -v TXBYTES=$TXBYTES -v COLLS=$COLLS -v RXERRORS=$RXERRORS -v TXERRORS=$TXERRORS  -F\  '/HWaddr/ && $1 == IF_NAME  {
	print  DATESTR " " IF_NAME " " RXBYTES " " TXBYTES " "  COLLS " " RXERRORS " " TXERRORS }' 

done
}

sadc_cpu_disp() {

# We have to answer with the following fields (separated by commas):
#	1: CPU_USER : percent user time
#	2: CPU_SYS : percent system time
#	3: CPU_WIO : percent wait for I/O time
#	4: FREEMEM : average free pages available to user processes
#	5: FREESWAP : disk blocks available for page swapping
#	6: RUNQSZ : run queue size
#	7: RUNOCC : run queue percent occupied - Not available
#	8: RCACHE : cache read hit ratio - Not available
#	9: WCACHE : cache write hit ratio - Not available
#	10: PGOUT : page out requests per second - Not available
#	11: PPGOUT : pages paged out per second
#	12: PGFREE : pages placed on the free list per second
#	13: PGSCAN : pages scanned per second - Not available
#	14: ATCH : page faults per second satisfied by reclaiming a page currently in memory - Not available
#	15: PGIN : page-in requests per second - Not available
#	16: PPGIN : pages paged-in per second
#	17: PGFLT : page faults per second from protection errors - Not available
#	18: VFLT : address translation page faults per second - Not available
#	19: SLOCK : faults per second caused by software lock requests requiring physical I/O - Not available
#	20: NPROC : number of processes
#	21: RPROC : number of processes running
#	22: BPROC : number of processes blocked
#	23: WPROC : number of processes waiting
#	24: DNLC - Not available
#	25: FORK : forks per second - Not available
#	26: EXEC : execs per second - Not available
RUNOCC=0
RCACHE=0
WCACHE=0
PGOUT=0
PGSCAN=0
ATCH=0
PGIN=0
PGFLT=0
VFLT=0
SLOCK=0
NPROC=`ps -ef|wc -l|sed -e 's/ *//'`
PROC_STATUS=`$VMSTATBIN 1 2 | tail -1 | $AWKBIN '{print $1","$2", 0"}'`
DNLC=0
FORK=0
EXEC=0
#if [ $OSRELEASE == "2.4" ]
#then
#	KAPM_IDLED=`ps -C kapm-idled -o pcpu | grep -v CPU`
#else
	KAPM_IDLED=0
#fi
$SARBIN -urWqR 1 | egrep -v '$^|AM|PM|\/s|\%|sz|Linux' | while read DUMMY DUMMY2 CPU_USER DUMMY3 CPU_SYS CPU_WIO
do
	read DUMMY PPGIN PPGOUT
        if [[ $PPGIN == "" ]]; then
             PPGIN=0
        fi
        if [[ $PPGOUT == "" ]]; then
             PPGOUT=0
        fi
	read DUMMY PGFREE DUMMY2 DUMMY3 DUMMY4
	if [[ ${PGFREE/.*/} -lt 0 || $PGFREE == "" ]]
	then
		PGFREE=0
	fi
	read DUMMY FREEMEM DUMMY2 DUMMY3 DUMMY4 DUMMY5 FREESWAP DUMMY6 DUMMY7 DUMMY8
        if [[ $FREEMEM == "" ]]; then
              FREEMEM=0
        fi
	read DUMMY RUNQSZ DUMMY2 DUMMY3 DUMMY4
        if [[ $RUNQSZ == "" ]]; then
              RUNQSZ=0
        fi
	CPU_SYS=$((${CPU_SYS/.*/}-${KAPM_IDLED/.*/}))
	if [[ $CPU_SYS -lt 0 || $CPU_SYS == "" ]]
	then
		CPU_SYS=0
	fi
        if [ $CPU_SYS -gt 100 ]
        then
                CPU_SYS=100
        fi
        CPU_USER=$((${CPU_USER/.*/}))
        if [[ $CPU_USER -lt 0 || $CPU_SYS == "" ]]; then
               CPU_USER=0
        fi 
        if [ $CPU_USER -gt 100 ]
        then 
                CPU_USER=100
        fi
        CPU_WIO=$((${CPU_WIO/.*/})) 
        if [[ $CPU_WIO -lt 0 || $CPU_WIO == "" ]]; then
                CPU_WIO=0
        fi       
        if [ $CPU_WIO -gt 100 ]
        then
                CPU_WIO=100
        fi
        # Use InActive memory because freemem is always the same on linux systems due to caching
         FREEMEM=`free | grep "^-" | awk -F\  ' { print $4 }'`

	echo "$DATESTR,$CPU_USER,$CPU_SYS,$CPU_WIO,$FREEMEM,$FREESWAP,$RUNQSZ,$RUNOCC,$RCACHE,$WCACHE,$PGOUT,$PPGOUT,$PGFREE,$PGSCAN,$ATCH,$PGIN,$PPGIN,$PGFLT,$VFLT,$SLOCK,$NPROC,$PROC_STATUS,$DNLC,$FORK,$EXEC"
	break
done
}

sadc_disk_disp() {
# We have to answer with the following fields (separated by spaces):
#	1: BUSY :  portion of time device was busy servicing  a  transfer request
#	2: AVQUE : average number of requests outstanding during that time
#	3: READS : number of read transfers  from  device
#	4: WRITES : number of write  transfers to  device
#	5: BLKS : number of bytes transferred in 512-byte units
#	6: AVWAIT: average wait time in milliseconds
#	7: AVSERV: average service time in milliseconds
#rm -f $SADC_DISKD_TMP $SADC_DISKDX_TMP
COUNTER=0
if [ $OSRELEASE == "2.2" ]
then
	$IOSTATBIN -d -x 1 2 | grep dev | $AWKBIN '{ print $0 }' > $SADC_DISKDX_TMP
	NUM_LINES=`cat $SADC_DISKDX_TMP | wc -l`
	cat $SADC_DISKDX_TMP | while read DEV RRQMS WRQMS RS WS RSECS WSECS AVGRQSZ AVGQUSZ AWAIT SVCTM UTIL
	do
		DEV=${DEV/*\/dev\//}
		if [ $COUNTER -ge $((NUM_LINES/2)) ] && [ ${DEV/[0-9]/} == $DEV ]
		then
		if [ "${RRQMS}" == "" ]; then
			# -- the line has been split
			read RRQMS WRQMS RS WS RSECS WSECS RKBS WKBS AVGRQSZ AVGQUSZ AWAIT SVCTM UTIL
		fi
			RWS=$((${RS/.*/}+${WS/.*/}))
                if [[ ${UTIL/.*/} -gt 100 ]]; then
                      UTIL=100
                fi
                        BLKSK_S=$((RKBS+WKBS))
			echo "$DATESTR $DEV $UTIL $AVGQUSZ $RWS $BLKSK_S $AWAIT $SVCTM"
		fi
		COUNTER=$((COUNTER+1))
	done
else
   shopt -s extglob
  $IOSTATBIN -d -x 1 2 | egrep -v '$^|Linux|Device' | $AWKBIN '{ print $0 }' > $SADC_DISKDX_TMP
   NUM_LINES=`cat $SADC_DISKDX_TMP | wc -l`
   cat $SADC_DISKDX_TMP | while read DEV RRQMS WRQMS RS WS RSECS WSECS RKBS WKBS AVGRQSZ AVGQUSZ AWAIT SVCTM UTIL
	do      
		#DEV=`echo $DEV | awk 'BEGIN { FS="/" } /^\// { split( $(NF), a, " " ); print a[1] }'`
        if [ $COUNTER -ge $((NUM_LINES/2)) ]
        then
                if [ "${RRQMS}" == "" ]; then
                        # -- the line has been split
                        read RRQMS WRQMS RS WS RSECS WSECS RKBS WKBS AVGRQSZ AVGQUSZ AWAIT SVCTM UTIL

                fi
		RS=""
                if [ ${DEV/part/} == $DEV ]
                then 
					if [ "$RS" == "" ]; then
  					  RS=0
					fi
					if [ "$WS" == "" ]; then
					  WS=0
					fi
                	RWS=$((${RS/.*/}+${WS/.*/}))
                if [[ ${UTIL/.*/} -gt 100 ]]; then
                      UTIL=100
                fi
                         BLKSK_S=$(echo "Results" | $AWKBIN -v RKBS=$RKBS -v WKBS=$WKBS '{print WKBS+RKBS }')
                	 echo "$DATESTR $DEV $UTIL $AVGQUSZ $RWS $BLKSK_S $AWAIT $SVCTM"
             	fi
        fi
         COUNTER=$((COUNTER+1))
   done
fi
}

mpstat_disp() {
# We have to answer with the following fields (separated by spaces):
#       1: CPUID : processor ID
#       2: MINFLT : minor faults
#       3: MAJFLT : major faults
#       4: XCAL : inter-processor cross-calls - Not available
#       5: INTR : interrupts
#       6: ITHR: interrupts as threads - Not available
#       7: CSW : context switches
#       8: ICSW : involuntary context switches - Not available
#       9: MIGR : thread migrations - Not available
#       10: SMTX : spins on mutexes - Not available
#       11: SRW : spins on reader/writer locks - Not available
#       12: SYSCL : system calls - Not available
#       13: CPU_USER : percent user time
#       14: CPU_SYS : percent system time
#       15: CPU_WIO : percent wait for I/O time
#       16: CPU_IDLE : percent idle time
CPUID=0
XCAL=0
ITHR=0
ICSW=0
CSW=0
MIGR=0
SMTX=0
SRW=0
SYSCL=0
#if [ $OSRELEASE == "2.4" ]
#then
#       KAPM_IDLED=`ps -C kapm-idled -o pcpu | grep -v CPU`
#else
        KAPM_IDLED=0
#fi
$SARBIN -x SELF -I SUM -P ALL -wu 1 | grep '^Average' >/tmp/$$.mp
INTR=`tail -3 /tmp/$$.mp | head -1 | awk '{ print $3 }'`
MINFLT=`tail -1 /tmp/$$.mp | awk '{ print $3 }'`
MAJFLT=`tail -1 /tmp/$$.mp | awk '{ print $4 }'`

tail -n +5 /tmp/$$.mp | while read DUMMY CPU_ID CPU_USER DUMMY CPU_SYS CPU_WIO CPU_IDLE
do
        if [ "$CPU_ID" == "INTR" ]; then
          break
        fi
 CPU_SYS=$((${CPU_SYS/.*/}-${KAPM_IDLED/.*/}))
        if [ $CPU_SYS -lt 0 ]
        then
                CPU_SYS=0
        fi
        CPU_IDLE=$((${CPU_IDLE/.*/}+${KAPM_IDLED/.*/}))
        if [ $CPU_IDLE -gt 100 ]
        then
                CPU_IDLE=100
        fi
        echo $DATESTR $CPU_ID $MINFLT $MAJFLT 0 $INTR 0 0 0 0 0 0 0 $CPU_USER $CPU_SYS $CPU_WIO $CPU_IDLE
done

rm /tmp/$$.mp

}

df_k_disp() {
# We answer the output of the df command in kilobytes but strip some filesystems like proc and shm that are not needed
echo $DATESTR
$DFBIN | egrep -v '/dev/shm|/proc|/dev/cdrom|/dev/fd0|/dev/pts|usb|Filesystem' | $AWKBIN '{  if ( $2 == "" ) { fs = $1; getline; print fs " " $0 } else { print $0 } } ' 
}

tcpinfo_disp() {
# We answer with a timestamp and the amount of retransmited packets per second between agent queries
NETSTR=`$NETSTATBIN -s | grep retransmited | $AWKBIN '{ print $1 }'`
if [ -f $TCPINFO_TMP ]
then
        PDATE=`cat $TMPDIR/.tcp.uptm | $AWKBIN '{ print $1 }'`
        if [ "$PDATE" == "" ]; then
             PDATE=$DATESTR
        fi
else
        PDATE=$DATESTR
fi
if [ -f $TCPINFO_TMP ]
then
        PVALUE=`cat $TMPDIR/.tcp.uptm | $AWKBIN '{ print $2 }'`
        if [ "$PVALUE" == "" ]; then
                PVALUE=$NETSTR
    fi
else
        PVALUE=$NETSTR
fi
DATEVAL=$((${DATESTR}-${PDATE}))
DATEVAL=$((${DATEVAL}/60+1))
TCPVAL=$((${NETSTR}-${PVALUE}))
TCPVAL=$((${TCPVAL}/${DATEVAL}))
echo $DATESTR " " $NETSTR > $TMPDIR/.tcp.uptm
if [ $TCPVAL -lt 0 ]
then
TCPVAL=0
fi
echo $DATESTR " " $TCPVAL
}

vxprint_disp() {
	echo ! 
}

metastat_disp() {
	echo ! 
}

prtmem_disp() {
	echo !
}

tailvaradm()
{
  logfile="/var/log/messages"
  default=10
  if [[ "$1" == "" && "$2" == "" ]] ; then
 	 tail -10 /var/log/messages
  fi
  if [[ "$1" != "" ]] ; then
	default=$1
  fi
  if [[ "$2" != "" ]] ; then
        logfile=$2
  fi
  tail -n $default $logfile
}
taillog()
{
  tail -n $2 $1
}

lastuser()
{
   last | head -10
}

whoin()
{
   echo $DATESTR
   who >$TMPDIR/$$.wx
   who | awk '{ print $1 }' | sort -u | while read id
   do
        C=`fgrep -c $id $TMPDIR/$$.wx`
	echo $id" "$C
   done

   rm -f $TMPDIR/$$.wx 2>/dev/null
}


chk4()
{
  PROC="$*"
#
# Use ps with format option to limit fields, then grep to narrow down the
# possibilities, then scripting to ensure
# you have actually mathces the process name, not the path or UID.
#
  TLIST=""
#  ps -eo "user pid args" | fgrep $PROC | fgrep -v fgrep | fgrep -v
# perfparse | while read ID PID PROC1 PARAMS
ps -eo "user pid args" | egrep -v "perfparse" | egrep -v "proc=" | $AWKBIN -v proc="$PROC" '

{
    checkproc=$0;
    regcheck = ".*" proc ".*"  ;
    if ( match(checkproc, regcheck ))
    {
        count = count +1;
        if (count == 1)
        {
            user = $1
            pid = $2
            process = $3
        }
    }
}

END {

if ( count > 0 )
 {
   print "OK - Found " count " " proc ;
 }
 else
 {
  print "CRIT - Count not find " proc ;
 }
}
'

}

vxvolstat_disp()
{
  echo "!"
}
lvm_list()
{
$UPTIME_DIR/checklvm.sh
}


case $1 in
	mpstat)		mpstat_disp
			exit
			;;
	netstat)	netstat_disp
			exit
			;;
	volcap)		df_k_disp
			exit
			;;
	tcpinfo)	tcpinfo_disp
			exit
			;;
	cpu)		sadc_cpu_disp
			exit
			;;
	disk)		sadc_disk_disp
			exit
			;;
	vxprint)	vxprint_disp
			exit
			;;
	metastat)	metastat_disp
			exit
			;;
	prtmem)		prtmem_disp
			exit
			;;
   	 chk4)         shift  1
                       chk4 "$*"
             	       exit
                       ;;
        taillog)            taillog $2 $3
                        exit
                        ;;
	tailvaradm)	tailvaradm $2 $3
			exit
			;;
	whoin)		whoin
			exit
			;;
	lvm_list)       lvm_list
			exit
			;;
	lastuser)	lastuser
			exit
			;;
	vxvolstat)     vxvolstat_disp
                        exit
			;;
		
	*)		help
			exit 1
			;;
esac
