#!/bin/bash
# $Id: sysinfo.sh,v 1.6 2008/01/21 17:55:53 dgermiquet Exp $

# This is the up.time linux agent
#
# CHANGELOG
# 25/09/2001 - Initial rev (Atha)
# 26/09/2001 - Implemented sysinfo, netstat functions (Atha)
# 27/09/2001 - Implemented mpstat, sadc_cpu, df-k functions (Atha)

TMPDIR="/opt/uptime-agent/tmp"
PATH=$PATH:/usr/bin:/usr/sbin
export PATH
SAR=/usr/local/bin/sar
if [ -x /bin/gawk ]; then
AWKBIN=/bin/gawk
else
  if [ -x /bin/awk ]; then
    AWKBIN=/bin/awk
  else
    AWKBIN=/usr/bin/awk
  fi
fi
TIMESTAMP=`date '+%Y%m%d%H%M'`
HOST=`uname -n`
LOGFILE=/opt/uptime-agent/logs/uptmagnt.log
log() {
	echo "$TIMESTAMP $1" >> $LOGFILE
}
#!/bin/sh
# Detects which OS and if it is Linux then it will detect which Linux Distribution.

OS=`uname -s`
REV=`uname -r`
MACH=`uname -m`

GetVersionFromFile()
{
	VERSION=`cat $1 | tr "\n" ' ' | sed s/.*VERSION.*=\ // `
}

if [ "${OS}" = "SunOS" ] ; then
	OS=Solaris
	ARCH=`uname -p`	
	OSSTR="${OS} ${REV}(${ARCH} `uname -v`)"
elif [ "${OS}" = "AIX" ] ; then
	OSSTR="${OS} `oslevel` (`oslevel -r`)"
elif [ "${OS}" = "Linux" ] ; then
	KERNEL=`uname -r`
	if [ -f /etc/redhat-release ] ; then
		DIST='RedHat'
		PSUEDONAME=`cat /etc/redhat-release | sed s/.*\(// | sed s/\)//`
		REV=`cat /etc/redhat-release | sed s/.*release\ // | sed s/\ .*//`
	elif [ -f /etc/SuSE-release ] ; then
		DIST=`cat /etc/SuSE-release | tr "\n" ' '| sed s/VERSION.*//`
		REV=`cat /etc/SuSE-release | tr "\n" ' ' | sed s/.*=\ //`
	elif [ -f /etc/mandrake-release ] ; then
		DIST='Mandrake'
		PSUEDONAME=`cat /etc/mandrake-release | sed s/.*\(// | sed s/\)//`
		REV=`cat /etc/mandrake-release | sed s/.*release\ // | sed s/\ .*//`
	elif [ -f /etc/debian_version ] ; then
		DIST="Debian `cat /etc/debian_version`"
		REV=""

	fi
	if [ -f /etc/UnitedLinux-release ] ; then
		DIST="${DIST}[`cat /etc/UnitedLinux-release | tr "\n" ' ' | sed s/VERSION.*//`]"
	fi
	
	OSSTR="${DIST} ${REV}(${PSUEDONAME} ${KERNEL} ${MACH})"

fi



echo "SYSNAME="`uname -n`
echo "DOMAIN="`domainname`
echo "ARCH=\""`uname -a`"\""
echo "OSVER=\"${OSSTR}\""
#echo "OSVER="`uname -r|$AWKBIN -F\- '{print $1}'`
NUMCPUS=`cat /proc/cpuinfo | grep -c processor`
echo "NUMCPUS="$NUMCPUS
echo "MEMSIZE="`cat /proc/meminfo | grep ^MemTotal | awk '{print $2}'`
PAGESIZE=`cat /proc/sys/vm/page-cluster`
echo "PAGESIZE"=$((PAGESIZE*1024))
echo "SWAPSIZE="`cat /proc/meminfo | grep SwapTotal | awk '{print $2}'`
echo "GPGSLO=0"
echo "VXVM=\"\""
echo "SDS=\"\""
TEMP=`/opt/uptime-agent/bin/pvs | grep "PV"`
if [  $? -ne 0 ] ; then
LVM=""
else 
LVM="Yes"
fi
echo "LVM=\"$LVM\""
echo "HOSTID="`hostid`
CPUARCH=`cat /proc/cpuinfo | egrep 'IA-64'`
if [ $? -ne 0  ]
then
  cat /proc/cpuinfo | egrep 'processor|model name|stepping|cpu MHz|cache size' | while read DATA
  do
        if [ ${DATA:0:4} = "proc" ]
        then
                CPU_NUM=`echo $DATA | awk '{print $3}'`
                read DATA
                MODEL=`echo $DATA | awk '{print $4$5}'`
                read DATA
                STEPPING=`echo $DATA | awk '{print $3}'`
                read DATA
                MHZ=`echo $DATA | awk '{print $4}'`
                read DATA
                CACHE=`echo $DATA | awk '{print $4}'`
                CACHE=$((CACHE/1024))
        fi
        echo "CPU${CPU_NUM}=\" ${CPU_NUM} 0 0 ${MHZ} $((CACHE/4)) ${MODEL} ${STEPPING} \""
  done
else
  cat /proc/cpuinfo | egrep 'processor|family|revision|cpu MHz' | while read DATA
  do
       if [ ${DATA:0:4} = "proc" ]
       then
                CPU_NUM=`echo $DATA | awk '{print $3}'`
                read DATA
                MODEL=`echo $DATA | awk '{print $3$4}'`
                read DATA
                REVISION=`echo $DATA | awk '{print $3}'`
                read DATA
                MHZ=`echo $DATA | awk '{print $4}'`
        fi
        echo "CPU${CPU_NUM}=\" ${CPU_NUM} 0 0 ${MHZ} 0 ${MODEL} ${REVISION} \""
  done
fi

IF_NUM=0
/sbin/ifconfig -a | egrep 'lo|eth|sit|inet |bond' | while read DATA
do
         echo "$DATA$" >> $TMPDIR/$$.net
done
flag=0;
cat $TMPDIR/$$.net | while read DATA
do
        echo "$DATA" | egrep "inet" >> /dev/null
        grepstatus=$?
        if [ $grepstatus -eq 1 ] ; then
	IF_NAME=`echo $DATA | awk '{print $1}'`
        flag=0;
        else 
	IF_IP=`echo $DATA | awk -F: '{print $2}' | awk '{ ip=$1; if ( ip == "" ) {ip="n/a"}; print ip}'`
        flag=1
        fi 
        if [ "$flag" == "1" ] ; then
        echo "NET$IF_NUM=$IF_NAME=$IF_IP"
        IF_NUM=$((IF_NUM+1))
        flag=0
        fi
done

# -- vmware
VMWARE=`/opt/uptime-agent/bin/vmw v`
if [ $? -ne 0 ]; then
  echo "VMWARE=0"
else
  echo "VMWARE=1"
  VMUUID=`/opt/uptime-agent/bin/vmw uuid | tr -d " -"`
  echo "VMUUID=$VMUUID"
fi


rm $TMPDIR/$$.net
